module.exports = {
  meta: {
    type: 'suggestion',
    docs: {
      description: '埋点方法必须包含基本属性'
    }
  },
  create: function (context) {
    function checkSaTrack(node) {
      if (['track', 'trackPageView', 'trackPage', 'trackClick', 'trackPageStop', 'trackAction'].includes(node.property.name)) {
        if (node.property.name === 'track') {
          checkArgumentAndAttr(node, 2)
        } else {
          checkArgumentAndAttr(node, 1)
        }
      }
    }
    function checkArgumentAndAttr(node, num) {
      const parent = node.parent
      if (parent.arguments && parent.arguments.length === num) {
        const argument = parent.arguments[num - 1]
        checkAttr(node, argument)
      } else {
        checkArgumentNum(node, num)
      }
    }
    function checkAttr(node, obj) {
      if (obj.type === 'ObjectExpression') {
        const keyArr = []
        const requireKeyArr = ['biz_name', 'requirement_id', 'track_sign', 'track_all_name']
        const lackKeyArr = []
        for (let item of obj.properties) {
          keyArr.push(item.key.name)
        }
        for (let item of requireKeyArr) {
          if (!keyArr.includes(item)) {
            lackKeyArr.push(item)
          }
        }
        if (lackKeyArr.length) {
          context.report({
            node: node,
            message: `缺少必填属性${lackKeyArr.join(', ')}`
          })
        }
      }
    }
    function checkArgumentNum(node, num) {
      context.report({
        node: node,
        message: `${node.property.name}方法必须有${num}个入参`
      })
    }
    return {
      MemberExpression: checkSaTrack
    }
  }
}
